#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

typedef struct disc_file_descriptor_t disc_file_descriptor_t;

#include "afs.h"
#include "memory.h"

void AdtMessageBox(const char* fmt, ...) {
	va_list args;

    va_start(args, fmt);
    vprintf(fmt, args);
    va_end(args);
}

disc_file_descriptor_t* DiscOpenFile(const char* path) {
	return (disc_file_descriptor_t*) fopen(path, "r");
}

int DiscReadFile(disc_file_descriptor_t* handle, void* buffer, int length) {
	fread(buffer, length, 1, (FILE*) handle);
	return 0;
}

int DiscSeekFile(disc_file_descriptor_t* handle, int offset, int whence) {
	int origin;
	switch (whence) {
		case 0:
			origin = SEEK_SET;
			break;
		case 1:
			origin = SEEK_CUR;
			break;
		case 2:
			origin = SEEK_END;
			break;
		default:
			abort();
	}

	return fseek((FILE*) handle, offset, origin);
}

static afs_volume_t vol;

int main(int argc, char* argv[]) {
	if (AfsOpenVolume(&vol, argv[1]) == -1) {
		exit(EXIT_FAILURE);
	}

	afs_file_descriptor_t* fd = AfsOpen(&vol, argv[2]);
	if (fd == 0) {
		exit(EXIT_FAILURE);
	}

	int length = AfsFileSize(&vol, fd);
	void* buffer = MemoryAllocate(length);
	AfsRead(&vol, fd, buffer, length);
	AfsClose(fd);

	write(1, buffer, length);
	MemoryFree(buffer);

	return EXIT_SUCCESS;
}
